package com.sd.business.service.production.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.production.dto.ProductionWorkOrderSelectDto;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.ProductionOrderScheduleVo;
import com.sd.business.entity.production.vo.ProductionWorkOrderVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.production.ProductionWorkOrderMapper;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.mq.enums.WorkOrderStatusEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 生产工单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-19
 */
@Service
public class ProductionWorkOrderServiceImpl extends ServiceImpl<ProductionWorkOrderMapper, ProductionWorkOrder> implements ProductionWorkOrderService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Override
    public Page<ProductionWorkOrderVo> getPage(ProductionWorkOrderSelectDto dto) {
        IWrapper<ProductionWorkOrder> wrapper = getWrapper();
        wrapper.orderByDesc("pwo", ProductionWorkOrder::getId);
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey());

        wrapper.like("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());

        wrapper.like("pwo", ProductionWorkOrder::getCode, dto.getProductionWorkOrderCode());

        wrapper.like("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.like("ss", SkuSpec::getName, dto.getSkuSpecName());

        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());

        wrapper.ge("oi", OrderInfo::getDeliveryTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getDeliveryTime, dto.getEndTime());

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public Map<Long, List<ProductionOrderScheduleVo>> getProductionOrderScheduleMap(List<Long> orderIdList) {
        if (orderIdList.isEmpty()) {
            return Collections.emptyMap();
        }

        IWrapper<Object> wrapper = IWrapper.getWrapper();
        wrapper.in("pwo", ProductionWorkOrder::getOrderId, orderIdList);
        wrapper.ne("pwo", ProductionWorkOrder::getStatus, WorkOrderStatusEnum.RESCHEDULE.getKey());
        wrapper.groupBy("pwo.order_sku_id");
        List<ProductionOrderScheduleVo> list = baseMapper.getProductionOrderScheduleList(wrapper);

        return list.stream().collect(Collectors.groupingBy(ProductionOrderScheduleVo::getOrderId));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(Long orderId) {
        OrderInfo orderInfo = orderService.getById(orderId);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }

        List<ProductionWorkOrder> productionWorkOrderList = new ArrayList<>();

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderId));

        String code = ObjectUtil.defaultIfNull(orderInfo.getWlnCode(), orderInfo.getCode());

        int num = 1;

        for (OrderSku orderSku : orderSkuList) {
            BigDecimal quantity = orderSku.getQuantity();
            for (int i = 0; i < quantity.intValue(); i++) {
                ProductionWorkOrder productionWorkOrder = new ProductionWorkOrder();
                productionWorkOrder.setCode(code + String.format("-%05d", num));
                productionWorkOrder.setOrderId(orderId);
                productionWorkOrder.setOrderSkuId(orderSku.getId());
                productionWorkOrder.setSkuSpecId(orderSku.getSkuSpecId());
                productionWorkOrder.setBomSpecId(orderSku.getBomSpecId());
                productionWorkOrder.setPrintType(orderSku.getPrintType());
                productionWorkOrder.setStatus(0);
                productionWorkOrderList.add(productionWorkOrder);
                num++;
            }
        }

        skuSpecService.attributeAssign(productionWorkOrderList, ProductionWorkOrder::getSkuSpecId, (item, skuSpec) -> {
            item.setMachinedPanel(skuSpec.getMachinedPanel());
        });

        saveBatch(productionWorkOrderList);
    }

}
